/*
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
*/



/********************************************/
/* Standard structure for datagrid elements */
/********************************************/

table.encommon_datagrid {
  border-width: .1rem .1rem .1rem .1rem;
  border-style: solid;
  border-color: rgba(var(--pretty), 1); }



/********************************************/
/* Extended structure for datagrid elements */
/********************************************/

table.encommon_datagrid
 >thead>tr>th {
  background: rgba(var(--gritty), .5);
  border-width: .1rem .1rem .1rem .1rem;
  border-style: solid;
  border-color: rgba(var(--pretty), 1);
  font-size: 1rem;
  font-weight: bold;
  line-height: 1rem;
  padding: .35rem .5rem .35rem .5rem; }

table.encommon_datagrid
 >tbody>tr>td {
  border-width: .1rem .1rem .1rem .1rem;
  border-style: solid;
  border-color: rgba(var(--pretty), 1);
  font-size: 1rem;
  line-height: 1rem;
  padding: .35rem .5rem .35rem .5rem;
  white-space: normal; }



/********************************************/
/* Extended structure for datagrid elements */
/********************************************/

table.encommon_datagrid
 >tbody>tr:nth-child(even)>td {
  background-color: rgba(var(--gritty), 0.125); }

table.encommon_datagrid
 >tbody>tr:hover>td {
  background-color: rgba(var(--pretty), 0.125); }
