"""
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Any



DictStrAny = dict[str, Any]
LDictStrAny = list[DictStrAny]



NCTrue = True
NCFalse = False
NCNone = None
