/*
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
*/



/********************************************/
/* Standard structure for colordiv elements */
/********************************************/

div.encommon_colordiv {
  display: inline-block; }



/********************************************/
/* Extended structure for colordiv elements */
/********************************************/

div.encommon_colordiv
 >._value {
  border-width: .2rem .2rem .2rem .2rem;
  border-style: solid;
  border-color: rgba(var(--foreground), 0.4);
  display: inline-block;
  border-radius: .5rem .5rem .5rem .5rem;
  height: 1.8rem;
  width: 1.8rem; }



/********************************************/
/* Extended structure for colordiv elements */
/********************************************/

div.encommon_colordiv
 >._label {
  background-color: rgba(var(--foreground), .2);
  border-radius: .2rem .2rem .2rem .2rem;
  display: inline-block;
  font-size: 1rem;
  line-height: 1rem;
  margin-left: .2rem;
  padding: .2rem .3rem .2rem .3rem; }



/********************************************/
/* Extended structure for datagrid elements */
/********************************************/

table.encommon_datagrid
 >tbody>tr>td:has(.encommon_colordiv) {
  text-align: center; }
