/*
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
*/



/********************************************/
/* Standard structure for moderate elements */
/********************************************/

div.encommon_moderate {
  display: inline-block; }



/********************************************/
/* Extended structure for moderate elements */
/********************************************/

div.encommon_moderate
 >._icon {
  display: table-cell; }


div.encommon_moderate
 >._icon>* {
  background-size: 1.5rem 1.5rem !important;
  height: 1.5rem !important;
  width: 1.5rem !important; }



/********************************************/
/* Extended structure for moderate elements */
/********************************************/

div.encommon_moderate
 >._value {
  display: table-cell; }

div.encommon_moderate
 >._value:not(:empty) {
  padding: 0 .2rem 0 .2rem; }



/********************************************/
/* Extended structure for moderate elements */
/********************************************/

div.encommon_moderate
 >._value>._label {
  font-size: 1rem;
  line-height: 1rem; }


div.encommon_moderate
 >._value>._small {
  font-size: .75rem;
  line-height: .75rem;
  opacity: 0.85; }
