/*
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
*/



/*********************************************/
/* Standard structure for datestamp elements */
/*********************************************/

div.encommon_datestamp {
  display: inline-block; }



/*********************************************/
/* Extended structure for datestamp elements */
/*********************************************/

div.encommon_datestamp
 >._value {
  font-family: monospace;
  font-size: 1rem;
  line-height: 1rem; }

div.encommon_datestamp
 >._delim {
  font-size: .9rem;
  line-height: .9rem;
  opacity: .75; }

div.encommon_datestamp
 >._tzname {
  font-family: monospace;
  font-size: .9rem;
  line-height: .9rem;
  opacity: .75; }



/********************************************/
/* Extended structure for datagrid elements */
/********************************************/

table.encommon_datagrid
 >tbody>tr>td:has(.encommon_datestamp) {
  text-align: center; }
