/*
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
*/



/*****************************************/
/* Standard structure for image elements */
/*****************************************/

.encommon_svgicon {
  height: 5rem;
  mask-repeat: no-repeat;
  mask-position: center;
  mask-size: contain;
  width: 5rem;
  -webkit-mask-repeat: no-repeat;
  -webkit-mask-position: center;
  -webkit-mask-size: contain; }



/*****************************************/
/* Extended structure for image elements */
/*****************************************/

.encommon_svgicon[data-image='failure'] {
  background-color: rgba(var(--color-red-lite), 1);
  mask-image: var(--svgicon-failure);
  -webkit-mask-image: var(--svgicon-failure); }

.encommon_svgicon[data-image='information'] {
  background-color: rgba(var(--color-blue-lite), 1);
  mask-image: var(--svgicon-information);
  -webkit-mask-image: var(--svgicon-information); }

.encommon_svgicon[data-image='success'] {
  background-color: rgba(var(--color-green-lite), 1);
  mask-image: var(--svgicon-success);
  -webkit-mask-image: var(--svgicon-success); }

.encommon_svgicon[data-image='warning'] {
  background-color: rgba(var(--color-yellow-lite), 1);
  mask-image: var(--svgicon-warning);
  -webkit-mask-image: var(--svgicon-warning); }
