/*
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
*/



/*******************************************/
/* Standard structure for numeric elements */
/*******************************************/

div.encommon_numeric {
  display: inline-block; }



/*******************************************/
/* Extended structure for numeric elements */
/*******************************************/

div.encommon_numeric
 >._value {
  font-family: monospace;
  font-size: 1rem;
  line-height: 1rem; }

div.encommon_numeric
 >._delim {
  font-size: .9rem;
  line-height: .9rem;
  opacity: .75; }

div.encommon_numeric
 >._decimal {
  font-family: monospace;
  font-size: .9rem;
  line-height: .9rem; }

div.encommon_numeric
 >._unit {
  font-size: .9rem;
  line-height: .9rem;
  opacity: .75; }



/********************************************/
/* Extended structure for datagrid elements */
/********************************************/

table.encommon_datagrid
 >tbody>tr>td:has(.encommon_numeric) {
  text-align: center; }
