/*
Functions and routines associated with Enasis Network Common Library.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
*/



/******************************************/
/* Standard structure for tagues elements */
/******************************************/

div.encommon_tagues {
  display: inline-block;
  white-space: normal; }



/******************************************/
/* Extended structure for tagues elements */
/******************************************/

div.encommon_tagues
 >._value {
  background-color: rgba(var(--gritty), .2);
  border-color: rgba(var(--pretty), .3);
  border-radius: .2rem .2rem .2rem .2rem;
  border-style: solid;
  border-width: .1rem .1rem .1rem .1rem;
  display: inline-block;
  font-family: monospace;
  font-size: .9rem;
  margin: .1rem .1rem .1rem .1rem;
  padding: .1rem .2rem .1rem .2rem; }



/********************************************/
/* Extended structure for datagrid elements */
/********************************************/

table.encommon_datagrid
 >tbody>tr>td:has(.encommon_tagues) {
  text-align: center; }
